/***************************************************************************/
/***  Name         : check_trans.sql                                     ***/
/***                                                                     ***/
/***  Description  : Validates ClearLogistics transitions.               ***/
/***                                                                     ***/
/***  Author       : First Choice Software, Inc                          ***/
/***                 4412 Spicewood Springs Road Suite 701               ***/
/***                 Austin, TX 78759                                    ***/
/***                                                                     ***/
/***  Usage        : First Choice Confidential                           ***/
/***                                                                     ***/
/***  Preconditions: None known                                          ***/
/***                                                                     ***/
/***  Postcondition: None known                                          ***/
/***                                                                     ***/
/***  Platforms    : This version supports SQL Server version 6.0 or     ***/
/***                  later.                                             ***/
/***                                                                     ***/
/***                                                                     ***/
/***  Copyright (C) 1997 First Choice Software, Inc.                     ***/
/***  All Rights Reserved                                                ***/
/***                                                                     ***/
/***************************************************************************/

/***************************************************************************/
/***     Header declarations.                                            ***/
/***************************************************************************/

use <db>
go

					/* Drop stored procedures, if      */
					/*  if defined                     */
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'check_cl_trans')
   DROP PROCEDURE check_cl_trans
go

/***************************************************************************/
/***     Check to see if a transition is legal for a given user.         ***/
/***************************************************************************/

CREATE PROCEDURE check_cl_trans @from_cond VARCHAR(80),
                                @to_cond VARCHAR(80),
                                @req_type VARCHAR(40),
                                @user_name VARCHAR(30),
                                @ret_val INT OUTPUT AS
BEGIN
  DECLARE @fc_upper VARCHAR(80),        /* From condition as upper case.   */
          @tc_upper VARCHAR(80),        /* To condition as upper case.     */
          @trans_mask VARCHAR(255),     /* Transition mask for privclass.  */
          @from_status VARCHAR(80),     /* Default status of from condition*/
          @to_status VARCHAR(80),       /* Default status of to condition. */
          @from_stat_objid INT,         /* Default status from cond objid. */
          @to_stat_objid INT,           /* Default status to cond objid.   */
          @t_pos INT,                   /* Position for scanning.          */
          @rank INT                     /* Rank of the trans in the mask.  */

                                        /* Convert conditions to upper case*/
  SELECT @fc_upper = UPPER(@from_cond)
  SELECT @tc_upper = UPPER(@to_cond)

                                        /* Strip off any suffixes.         */
  SELECT @t_pos = CHARINDEX('-', @fc_upper)
  IF @t_pos > 0
  BEGIN
     SELECT @from_cond = SUBSTRING(@from_cond, 1, @t_pos - 1)
  END
  SELECT @t_pos = CHARINDEX('-', @tc_upper)
  IF @t_pos > 0
  BEGIN
     SELECT @to_cond = SUBSTRING(@to_cond, 1, @t_pos - 1)
  END

                                        /* Change open/closed conditions as*/
                                        /*  needed.                        */
  IF @fc_upper = 'OPEN'
  BEGIN
     SELECT @from_cond = 'Rqst Open'
  END
  IF @fc_upper = 'CLOSED'
  BEGIN
     SELECT @from_cond = 'Rqst Closed'
  END
  IF @tc_upper = 'OPEN'
  BEGIN
     SELECT @to_cond = 'Rqst Open'
  END
  IF @tc_upper = 'CLOSED'
  BEGIN
     SELECT @to_cond = 'Rqst Closed'
  END

                                        /* Get default status for from     */
                                        /*  from condition. If error, leave*/
  EXEC get_list_item @from_cond, "", @from_status OUT, @from_stat_objid OUT
  IF @from_status IS NULL OR @from_status = ''
  BEGIN
     SELECT @ret_val = -1
     GOTO end_proc
  END

                                        /* Get default status for to       */
                                        /*  from condition. If error, leave*/
  EXEC get_list_item @to_cond, "", @to_status OUT, @to_stat_objid OUT
  IF @to_status IS NULL OR @to_status = ''
  BEGIN
     SELECT @ret_val = -1
     GOTO end_proc
  END
                                        /* Get transition rank. If not     */
                                        /*  found, error and leave.        */
  SELECT @rank = rank FROM table_transition
    WHERE focus_type = 49 AND focus_subtype = @req_type AND
          from_state2gbst_elm = @from_stat_objid AND
          to_state2gbst_elm = @to_stat_objid
  IF @rank IS NULL
  BEGIN
     SELECT @ret_val = -2
     GOTO end_proc
  END

                                        /* Get trans mask from privclass   */
                                        /*  the user.                      */
  SELECT @trans_mask = trans_mask FROM table_privclass 
    WHERE objid = (SELECT user_access2privclass FROM table_user 
                          WHERE login_name = @user_name)

                                        /* If user can't transition, error.*/
                                        /* Else, all is OK.                */
  IF SUBSTRING(@trans_mask, @rank + 1, 1) = "0"
  BEGIN
     SELECT @ret_val = -3
  END
  ELSE
  BEGIN
     SELECT @ret_val = 0
  END
end_proc:
END
go

/***************************************************************************/
/***     Permission granting statements.                                 ***/
/***************************************************************************/

GRANT EXECUTE ON check_cl_trans TO PUBLIC
go

/***************************************************************************/
/***     Test routine.                                                   ***/
/***************************************************************************/

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'test')
   DROP PROCEDURE test
go

CREATE PROCEDURE test AS
BEGIN
  DECLARE @ret_val INT,
          @t_str VARCHAR(100)

  EXEC check_cl_trans 'Picked-Rejected', 'Unpicked-Forward', 'Please Specify', 'sa', @ret_val OUT
  SELECT @t_str = "Return code: " + CONVERT(CHAR, @ret_val)
  PRINT @t_str
END
go

GRANT EXECUTE ON test TO PUBLIC
go

quit